@@ -1,29 +1,25 @@
+use log;
 use std::fmt;
 use std::error::Error;
 use crate::utils;
-use crate::source::ModuleSource;
 use crate::lexer::{Span, TokenMeta, LexerError};
+use crate::debug::SourceError;
 use crate::debug::symbol::{DebugSymbol, TokenIndex};
 
 
-pub type ErrorKind = ParserErrorKind;
+pub type ParseResult<T> = Result<T, ParserError>;
 
-// Specifies the actual error that occurred
 #[derive(Debug)]
-pub enum ParserErrorKind {
+pub enum ErrorKind {
     LexerError,
     EndofTokenStream,
-    ExpectedStartOfExpr,   // expected the start of an expression
-    ExpectedCloseParen,
-    ExpectedCloseSquare,
-    ExpectedCloseBrace,
-    ExpectedIdentifier,
-    ExpectedItemAfterLabel,   // expected either begin, while, for after label
-    InvalidAssignment,      // the LHS of an assignment was not a valid lvalue
-    InvalidDeclAssignment,  // only "=" allowed in variable decls
-    ControlFlowOutsideOfBlock,
-    ExpectedEndAfterControlFlow,
-    DeclMissingInitializer, 
+    SyntaxError(String),
+}
+
+impl<S> From<S> for ErrorKind where S: ToString {
+    fn from(message: S) -> Self {
+        ErrorKind::SyntaxError(message.to_string())
+    }
 }
 
 // Provide information about the type of syntactic construct from which the error originated
@@ -37,8 +33,12 @@ pub enum ContextTag {
     ControlFlow,
     ExprMeta,
     BlockExpr,
+    IfExpr,
+    FunDefStmt,
+    FunDefExpr,
+    FunParam,
+    DeclarationExpr,
     AssignmentExpr,
-    VarDeclExpr,
     BinaryOpExpr,
     UnaryOpExpr,
     PrimaryExpr,
@@ -51,95 +51,95 @@ pub enum ContextTag {
     Label,
 }
 
-// Since ErrorContext can share references with the Parser, we need to use 
-// an error type that does not refer to the error context internally.
-// The error context is always available at the base of the recursive descent call stack and can be added later.
-#[derive(Debug)]
-pub struct ErrorPrototype {
-    kind: ErrorKind,
-    lexer_error: Option<LexerError>,
-}
-
-impl ErrorPrototype {
-    pub fn kind(&self) -> &ErrorKind { &self.kind }
-}
-
-impl From<ParserErrorKind> for ErrorPrototype {
-    fn from(kind: ParserErrorKind) -> Self {
-        ErrorPrototype { kind, lexer_error: None }
+impl From<ErrorKind> for ParserError {
+    fn from(kind: ErrorKind) -> Self {
+        Self { 
+            kind, context: None, symbol: None, cause: None,
+        }
     }
 }
 
-impl From<LexerError> for ErrorPrototype {
+impl From<&str> for ParserError {
+    fn from(message: &str) -> Self {
+        Self { 
+            kind: message.into(), 
+            context: None, symbol: None, cause: None,
+        }
+    }
+}
+
+impl From<LexerError> for ParserError {
     fn from(error: LexerError) -> Self {
-        ErrorPrototype { 
+        Self { 
             kind: ErrorKind::LexerError, 
-            lexer_error: Some(error) 
+            context: None,
+            symbol: Some((&error.span).into()),
+            cause: Some(Box::new(error)),
         }
     }
 }
 
 #[derive(Debug)]
-pub struct ParserError<'m> {
+pub struct ParserError {
     kind: ErrorKind,
-    module: &'m ModuleSource,
-    context: ContextTag,
-    symbol: DebugSymbol,
+    context: Option<ContextTag>,
+    symbol: Option<DebugSymbol>,
     cause: Option<Box<dyn Error>>,
 }
 
-impl<'m> ParserError<'m> {
-    pub fn from_prototype(proto: ErrorPrototype, context: ErrorContext<'m>) -> Self {
-        let module = context.module;
-        let context_tag = context.frame().context();
-        
-        let symbol;
-        let cause;
-        if let Some(error) = proto.lexer_error {
-            symbol = (&error.span).into();
-            cause = Some(Box::new(error) as Box<dyn Error>);
-        } else {
-            symbol = context.take_debug_symbol();
-            cause = None;
+impl ParserError {
+    pub fn with_context_tag(mut self, context: ContextTag) -> Self {
+        self.context.replace(context); self
+    }
+    
+    pub fn with_symbol(mut self, symbol: DebugSymbol) -> Self {
+        self.symbol.replace(symbol); self
+    }
+    
+    pub fn with_symbol_from_ctx(mut self, ctx: &ErrorContext) -> Self {
+        if let Some(symbol) = ctx.frame().as_debug_symbol() {
+            self.symbol.replace(symbol);
         }
-        
-        ParserError {
-            module, cause, symbol,
-            kind: proto.kind,
-            context: context_tag,
+        self
+    }
+    
+    pub fn with_cause(mut self, error: impl Error + 'static) -> Self {
+        self.cause.replace(Box::new(error)); self
+    }
+    
+    // fill in fields from context if not already set
+    pub fn with_error_context(mut self, context: ErrorContext) -> Self {
+        if self.context.is_none() {
+            self.context.replace(context.frame().context());
         }
+        if self.symbol.is_none() {
+            self.symbol.replace(context.take_debug_symbol());
+        }
+        self
     }
     
     pub fn kind(&self) -> &ErrorKind { &self.kind }
-    pub fn module(&self) -> &ModuleSource { self.module }
-    pub fn context(&self) -> &ContextTag { &self.context }
-    pub fn debug_symbol(&self) -> &DebugSymbol { &self.symbol }
+    pub fn context(&self) -> Option<&ContextTag> { self.context.as_ref() }
 }
 
 
-impl Error for ParserError<'_> {
+impl Error for ParserError {
     fn source(&self) -> Option<&(dyn Error + 'static)> {
         self.cause.as_ref().map(|o| o.as_ref())
     }
 }
 
-impl fmt::Display for ParserError<'_> {
+impl SourceError for ParserError {
+    fn debug_symbol(&self) -> Option<&DebugSymbol> { self.symbol.as_ref() }
+}
+
+impl fmt::Display for ParserError {
     fn fmt(&self, fmt: &mut fmt::Formatter<'_>) -> Result<(), fmt::Error> {
         
         let message = match self.kind() {
             ErrorKind::LexerError => "",
             ErrorKind::EndofTokenStream => "unexpected end of token stream",
-            ErrorKind::ExpectedStartOfExpr  => "expected an expression here",
-            ErrorKind::ExpectedCloseParen   => "missing closing ')'",
-            ErrorKind::ExpectedCloseSquare  => "missing closing ']'",
-            ErrorKind::ExpectedCloseBrace   => "missing closing '}'",
-            ErrorKind::ExpectedIdentifier   => "invalid identifier",
-            ErrorKind::InvalidAssignment    => "invalid assignment",
-            ErrorKind::InvalidDeclAssignment  => "only '=' is allowed when initializing a newly declared variable",
-            ErrorKind::DeclMissingInitializer => "missing '=' initializer for variable declaration",
-            ErrorKind::ExpectedItemAfterLabel => "expected loop statement or block expression after label",
-            ErrorKind::ControlFlowOutsideOfBlock => "found 'return', 'break', or 'continue' outside of a block",
-            ErrorKind::ExpectedEndAfterControlFlow => "'return', 'break', or 'continue' must be the last statement in a block",
+            ErrorKind::SyntaxError(message) => message,
         };
         
         utils::format_error(fmt, "syntax error", Some(message), self.source())
@@ -149,33 +149,41 @@ impl fmt::Display for ParserError<'_> {
 
 // Structures used by the parser for error handling and synchronization
 
-#[derive(Debug, Clone)]
-pub struct ErrorContext<'m> {
-    module: &'m ModuleSource,
+#[derive(Debug)]
+pub struct ErrorContext {
     stack: Vec<ContextFrame>,
 }
 
-impl<'m> ErrorContext<'m> {
-    pub fn new(module: &'m ModuleSource, base: ContextTag) -> Self {
+impl<'m> ErrorContext {
+    pub fn new(base: ContextTag) -> Self {
         ErrorContext {
-            module, stack: vec![ ContextFrame::new(base) ],
+            stack: vec![ ContextFrame::new(base) ],
         }
     }
     
-    //pub fn module(&self) -> &'m str { self.module }
+    pub fn frame(&self) -> &ContextFrame { 
+        self.stack.last().unwrap() 
+    }
     
-    pub fn frame(&self) -> &ContextFrame { self.stack.last().unwrap() }
-    pub fn frame_mut(&mut self) -> &mut ContextFrame { self.stack.last_mut().unwrap() }
+    pub fn frame_mut(&mut self) -> &mut ContextFrame { 
+        self.stack.last_mut().unwrap() 
+    }
     
-    pub fn push(&mut self, tag: ContextTag) { self.stack.push(ContextFrame::new(tag)) }
+    pub fn push(&mut self, tag: ContextTag) { 
+        log::debug!("Push frame: {:0>3} {:?}", self.stack.len()+1, tag);
+        self.stack.push(ContextFrame::new(tag)) 
+    }
     
-    pub fn push_continuation(&mut self, tag: ContextTag) {
-        let start = self.frame().start().map(|o| o.to_owned());
+    // treat the prev top frame as if it was *above* the new top frame
+    pub fn push_continuation(&mut self, tag: ContextTag, frame: Option<ContextFrame>) {
+        // log::debug!("Set frame:  {:0>3} {:?} -> {:?}", self.stack.len(), self.frame().context(), tag);
+        let frame = frame.unwrap_or_else(|| self.frame().clone());
         self.push(tag);
-        self.frame_mut().set_span(start, None);
+        self.frame_mut().extend(frame);
     }
     
     pub fn pop(&mut self) -> ContextFrame { 
+        log::debug!("Pop frame:  {:0>3} {:?}", self.stack.len(), self.frame().context());
         assert!(self.stack.len() > 1);
         self.stack.pop().unwrap()
     }
@@ -234,11 +242,11 @@ impl ContextFrame {
     pub fn end(&self) -> Option<&Span> { self.end.as_ref() }
     
     pub fn set_start(&mut self, token: &TokenMeta) { 
-        self.start.replace(token.span.clone()); 
+        self.start.replace(token.span); 
     }
     
     pub fn set_end(&mut self, token: &TokenMeta) { 
-        self.end.replace(token.span.clone()); 
+        self.end.replace(token.span); 
     }
     
     pub fn set_span(&mut self, start: Option<Span>, end: Option<Span>) {
@@ -247,25 +255,19 @@ impl ContextFrame {
     }
     
     pub fn extend(&mut self, other: ContextFrame) {
-        if self.start.as_ref().and(other.start.as_ref()).is_some() {
-            if span_lt(other.start.as_ref().unwrap(), self.start.as_ref().unwrap()) {
-                self.start = other.start;
-            }
-        } else if other.start.is_some() {
-            self.start = other.start;
-        }
+        let spans = [self.start, other.start, self.end, other.end];
+        let start = spans.iter().filter_map(|s| s.as_ref())
+            .min_by(|a, b| a.start_index().cmp(&b.start_index()));
         
-        if self.end.as_ref().and(other.end.as_ref()).is_some() {
-            if span_lt(self.end.as_ref().unwrap(), other.end.as_ref().unwrap()) {
-                self.end = other.end;
-            }
-        } else if other.end.is_some() {
-            self.end = other.end;
-        }
+        let end = spans.iter().filter_map(|s| s.as_ref())
+            .max_by(|a, b| a.end_index().cmp(&b.end_index()));
+        
+        self.start = start.map(|s| s.clone());
+        self.end = end.map(|s| s.clone());
     }
     
     pub fn as_debug_symbol(&self) -> Option<DebugSymbol> {
-        match (self.start.clone(), self.end.clone()) {
+        match (self.start, self.end) {
             
             (Some(start), Some(end)) => {
                 let start_index = start.index;
