@@ -1,11 +1,12 @@
-use crate::debug::symbol::DebugSymbol;
+use crate::debug::DebugSymbol;
 use crate::runtime::types::operator::{BinaryOp, UnaryOp};
 use crate::parser::primary::{Atom, Primary};
 use crate::parser::assign::{Assignment, Declaration};
+use crate::parser::fundefs::FunctionDef;
 use crate::parser::structs::{ObjectConstructor};
 use crate::parser::stmt::{StmtMeta, Stmt, Label};
 
-
+// TODO replace Vecs with boxed slices
 #[derive(Debug, Clone)]
 pub enum Expr {
     
@@ -20,22 +21,34 @@ pub enum Expr {
     Assignment(Box<Assignment>), // box the whole Assignment (instead of just lhs Expr) to keep size of ExprMeta down
     
     Declaration(Box<Declaration>),
-
-    Tuple(Vec<ExprMeta>),
     
-    ObjectCtor(Box<ObjectConstructor>),
+    Tuple(Box<[ExprMeta]>),
     
-    // IfExpr
+    // ObjectCtor(Box<ObjectConstructor>),
     
-    Block(Vec<StmtMeta>, Option<Label>), // TODO label
+    IfExpr(IfExpr),
+    
+    Block(Option<Label>, Box<[StmtMeta]>),
+    
+    FunctionDef(FunctionDef),
     
-    // FunctionDef
     // ClassDef
 }
 
+#[derive(Debug, Clone)]
+pub struct IfExpr {
+    pub branches: Box<[CondBranch]>,
+    pub else_branch: Option<Box<[StmtMeta]>>,
+}
 
-// Expr + DebugSymbol
+#[derive(Debug, Clone)]
+pub struct CondBranch {
+    pub cond: Expr,
+    pub suite: Box<[StmtMeta]>,
+}
 
+
+/// An `Expr` plus a `DebugSymbol`
 #[derive(Debug, Clone)]
 pub struct ExprMeta {
     variant: Expr,
