@@ -1,5 +1,5 @@
 use crate::language;
-use crate::runtime::strings::{InternSymbol, StringInterner};
+use crate::runtime::strings::InternSymbol;
 use crate::parser::expr::{ExprMeta, Expr};
 use crate::parser::structs::ObjectConstructor;
 
@@ -10,8 +10,8 @@ use crate::parser::structs::ObjectConstructor;
 pub enum Atom {
     Nil,
     EmptyTuple,
-    Self_,
-    Super,
+    // Self_,
+    // Super,
     Identifier(InternSymbol),
     BooleanLiteral(bool),
     IntegerLiteral(language::IntType),
@@ -26,7 +26,7 @@ pub enum AccessItem {
     Attribute(InternSymbol),
     Index(ExprMeta),
     Invoke(),       // TODO
-    Construct(ObjectConstructor),
+    // Construct(ObjectConstructor),
 }
 
 #[derive(Debug, Clone)]
