@@ -1,5 +1,5 @@
 use crate::runtime::strings::InternSymbol;
-use crate::debug::symbol::DebugSymbol;
+use crate::debug::DebugSymbol;
 use crate::parser::expr::Expr;
 
 
@@ -26,7 +26,7 @@ pub enum Stmt {
     Return(Option<Expr>),
     
     Echo(Expr),
-    
+    Assert(Expr),
 }
 
 impl Stmt { }
